import * as React from 'react';
import { useEffect, useState, Dispatch, SetStateAction } from 'react';
import { Button, Box, Typography, IconButton } from '@mui/material';
import AddIcon from '@mui/icons-material/Add';
import DeleteIcon from '@mui/icons-material/Delete';
import UploadFileIcon from '@mui/icons-material/UploadFile';
import UploadDialog from './UploadDialog'; // Ensure the file extension is correct
import { SxProps } from '@mui/material';
import bottleGrey from '../assets/icons/bottle-svgrepo-com-grey.svg';
import bottleYellow from '../assets/icons/bottle-svgrepo-com-yellow.svg';
import bottleGreen from '../assets/icons/bottle-svgrepo-com-green.svg';
import bottleRed from '../assets/icons/bottle-svgrepo-com-red.svg';
import { Shipment_Input, DefaultService, OpenAPI } from "../../openapi";

interface ShipmentPanelProps {
    selectedPage?: string;
    setIsCreatingShipment: Dispatch<SetStateAction<boolean>>;
    selectShipment: (shipment: Shipment_Input | null) => void;
    sx?: SxProps;
}

const ShipmentPanel: React.FC<ShipmentPanelProps> = ({
                                                         setIsCreatingShipment,
                                                         selectShipment,
                                                         sx,
                                                     }) => {
    const [shipments, setShipments] = useState<Shipment_Input[]>([]);
    const [selectedShipment, setSelectedShipment] = useState<Shipment_Input | null>(null);
    const [error, setError] = useState<string | null>(null);
    const [uploadDialogOpen, setUploadDialogOpen] = useState(false);

    const fetchShipments = async () => {
        console.log('trying to fetch some shipments');
        try {
            const s: Shipment_Input[] = await DefaultService.getShipmentsShipmentsGet();
            setShipments(s);
        } catch (error) {
            console.error('Failed to fetch shipments:', error);
            setError("Failed to fetch shipments. Please try again later.");
        }
    };

    useEffect(() => {
        OpenAPI.BASE = 'http://127.0.0.1:8000';
        fetchShipments();
    }, []);

    const statusIconMap: Record<string, string> = {
        "In Transit": bottleYellow,
        "Delivered": bottleGreen,
        "Pending": bottleGrey,
        "Unknown": bottleRed,
    };

    const handleShipmentSelection = (shipment: Shipment_Input) => {
        const isCurrentlySelected = selectedShipment?.shipment_id === shipment.shipment_id;
        setSelectedShipment(isCurrentlySelected ? null : shipment);
        selectShipment(isCurrentlySelected ? null : shipment);
    };

    const handleDeleteShipment = () => {
        if (selectedShipment) {
            const confirmed = window.confirm(`Are you sure you want to delete the shipment: ${selectedShipment.shipment_name}?`);
            if (confirmed) {
                const updatedShipments = shipments.filter(shipment => shipment.shipment_id !== selectedShipment.shipment_id);
                setShipments(updatedShipments);
                setSelectedShipment(null);
            }
        }
    };

    const handleOpenUploadDialog = () => {
        setUploadDialogOpen(true);
    };

    const handleCloseUploadDialog = () => {
        setUploadDialogOpen(false);
    };

    return (
        <Box sx={{ width: '90%', borderRight: '1px solid #ccc', padding: 2, ...sx }}>
            {error && <Typography color="error">{error}</Typography>}
            <Typography variant="h6" sx={{ marginBottom: 2, fontWeight: 'bold' }}>
                Shipments
            </Typography>
            <Button
                variant="contained"
                color="primary"
                startIcon={<AddIcon />}
                onClick={() => setIsCreatingShipment(true)}
                sx={{ marginBottom: 2, padding: '10px 16px' }}
            >
                Create Shipment
            </Button>

            {shipments.map((shipment) => (
                <Button
                    key={shipment.shipment_id}
                    onClick={() => handleShipmentSelection(shipment)}
                    sx={{
                        width: '100%',
                        textAlign: 'left',
                        marginBottom: 1,
                        color: 'white',
                        whiteSpace: 'nowrap',
                        overflow: 'hidden',
                        textOverflow: 'ellipsis',
                        padding: '10px 16px',
                        fontSize: '0.7rem',
                        display: 'flex',
                        alignItems: 'center',
                        justifyContent: 'space-between',
                        backgroundColor: selectedShipment?.shipment_id === shipment.shipment_id ? '#52893e' : '#424242',
                        '&:hover': {
                            backgroundColor: selectedShipment?.shipment_id === shipment.shipment_id ? '#9aca8c' : '#616161',
                        },
                        '&:active': {
                            backgroundColor: selectedShipment?.shipment_id === shipment.shipment_id ? '#915151' : '#212121',
                        },
                    }}
                >
                    <div style={{ display: 'flex', alignItems: 'center' }}>
                        <div style={{ position: 'relative', marginRight: '8px' }}>
                            <img
                                src={statusIconMap[shipment.shipment_status] || bottleGrey}
                                alt={`Status: ${shipment.shipment_status}`}
                                width="24"
                            />
                            <span style={{
                                position: 'absolute',
                                top: '0%',
                                right: '0%',
                                transform: 'translate(50%, -50%)',
                                color: 'white',
                                fontWeight: 'bold',
                                fontSize: '0.6rem',
                                backgroundColor: 'transparent',
                                borderRadius: '50%',
                                padding: '0 2px',
                            }}>
                                {shipment.number_of_dewars}
                            </span>
                        </div>
                        <div>
                            <div>{shipment.shipment_name}</div>
                            <div style={{ fontSize: '0.6rem', color: '#ccc' }}>{shipment.shipment_date}</div>
                            <div style={{ fontSize: '0.6rem', color: '#ccc' }}>
                                Total Pucks: {shipment.dewars.reduce((total, dewar) => total + dewar.number_of_pucks, 0)}
                            </div>
                        </div>
                    </div>
                    <div style={{ display: 'flex', alignItems: 'center' }}>
                        <IconButton
                            onClick={handleOpenUploadDialog}
                            color="primary"
                            title="Upload Sample Data Sheet"
                            sx={{ marginLeft: 1 }}
                        >
                            <UploadFileIcon />
                        </IconButton>
                        {selectedShipment?.shipment_id === shipment.shipment_id && (
                            <IconButton
                                onClick={handleDeleteShipment}
                                color="error"
                                title="Delete Shipment"
                                sx={{ marginLeft: 1 }}
                            >
                                <DeleteIcon />
                            </IconButton>
                        )}
                    </div>
                </Button>
            ))}
            <UploadDialog
                open={uploadDialogOpen}
                onClose={handleCloseUploadDialog}
            />
        </Box>
    );
};

export default ShipmentPanel;
