import React, { useState, useEffect } from 'react';
import { Grid } from '@mui/material';
import ShipmentPanel from '../components/ShipmentPanel.tsx';
// import ShipmentDetails from '../components/ShipmentDetails.tsx'; // Commented out for now
import ShipmentForm from '../components/ShipmentForm.tsx';
import { ContactPerson, Address, Proposal, Dewar, DefaultService, Shipment_Input } from '../../openapi'; // Import your API service

const ShipmentView: React.FC = () => {
    // State for managing shipments
    const [shipments, setShipments] = useState<Shipment_Input[]>([]);
    const [newShipment, setNewShipment] = useState<Shipment_Input>({
        shipment_name: '',
        contact_person: [],
        proposal_number: '',
        return_address: [],
        comments: '',
        dewars: [],
    });
    const [isCreatingShipment, setIsCreatingShipment] = useState(false);
    const [selectedShipment, setSelectedShipment] = useState<Shipment_Input | null>(null);
    const [selectedDewar, setSelectedDewar] = useState<Dewar | null>(null);
    const [contactPersons, setContactPersons] = useState<ContactPerson[]>([]);
    const [returnAddresses, setReturnAddresses] = useState<Address[]>([]);
    const [proposals, setProposals] = useState<Proposal[]>([]);

    // Fetch initial shipments data
    const fetchShipments = async () => {
        try {
            const fetchedShipments = await DefaultService.getShipmentsShipmentsGet();
            setShipments(fetchedShipments);
        } catch (error) {
            console.error("Failed to fetch shipments:", error);
        }
    };

    useEffect(() => {
        fetchShipments();
    }, []);

    // Handle saving a new shipment
    const handleSaveShipment = async (shipment: Shipment_Input) => {
        try {
            const savedShipment = await DefaultService.createShipmentShipmentsPost(shipment);
            // Update the shipments list with the newly created shipment
            setShipments(prev => [...prev, savedShipment]); // Add the new shipment to the state
            onShipmentCreated(savedShipment); // Trigger the callback after saving
        } catch (error) {
            console.error("Failed to save shipment:", error);
        }
    };

    // Shipment creation callback
    const onShipmentCreated = async (newShipment: Shipment_Input) => {
        setIsCreatingShipment(false); // Close the form
        console.log("Shipment created:", newShipment);

        // Reset the form
        setNewShipment({
            shipment_name: '',
            contact_person: [],
            proposal_number: [],
            return_address: [],
            comments: '',
            dewars: [],
        });
        await fetchShipments(); // Re-fetch the shipments after creation
    };

    // Function to select a shipment
    const selectShipment = (shipment: Shipment_Input | null) => {
        setSelectedShipment(shipment);
    };

    return (
        <Grid container spacing={2} sx={{ height: '100vh' }}>
            {/* Left column: ShipmentPanel */}
            <Grid
                item
                xs={12}
                md={3}
                sx={{
                    display: 'flex',
                    flexDirection: 'column',
                    flexGrow: 0,
                }}
            >
                <ShipmentPanel
                    selectedPage="Shipments"
                    setIsCreatingShipment={setIsCreatingShipment}
                    fetchShipments={fetchShipments}
                    shipments={shipments} // Pass down the updated shipments
                    selectShipment={selectShipment}
                />
            </Grid>

            {/* Right column: ShipmentForm or ShipmentDetails */}
            <Grid
                item
                xs={12}
                md={9}
                sx={{
                    display: 'flex',
                    flexDirection: 'column',
                    flexGrow: 1,
                }}
            >
                {isCreatingShipment ? (
                    <ShipmentForm
                        newShipment={newShipment}
                        setNewShipment={setNewShipment}
                        handleSaveShipment={handleSaveShipment}
                        onShipmentCreated={onShipmentCreated} // Pass this prop
                        contactPersons={contactPersons}
                        proposals={proposals}
                        returnAddresses={returnAddresses}
                        sx={{ flexGrow: 1 }} // Allow form to grow and take available space
                    />
                ) : (
                    // Commented out ShipmentDetails for now
                    // <ShipmentDetails
                    //     selectedShipment={selectedShipment}
                    //     setSelectedDewar={setSelectedDewar}
                    //     isCreatingShipment={isCreatingShipment}
                    //     newShipment={newShipment}
                    //     setNewShipment={setNewShipment}
                    //     handleSaveShipment={handleSaveShipment}
                    //     contactPersons={contactPersons}
                    //     proposals={proposals}
                    //     returnAddresses={returnAddresses}
                    //     sx={{ flexGrow: 1 }} // Allow details to grow and take available space
                    // />
                    <div>No shipment details available.</div> // Placeholder while ShipmentDetails is commented out
                )}
            </Grid>
        </Grid>
    );
};

export default ShipmentView;
